@@ -2,50 +2,42 @@ import collections.abc
 
 from math import isnan
 from statistics import mean, median, stdev, mode
+from abc import abstractmethod, ABC
 from numbers import Number
 from collections import defaultdict
 from itertools import islice, chain
 from typing import Hashable, Optional, Sequence, Union, Iterable, Dict, Any, List, Tuple
-from coba.backport import Literal
+from coba.backports import Literal
 
 from coba import pipes
 from coba.random import CobaRandom
+from coba.exceptions import CobaException
 from coba.statistics import iqr
 
-from coba.environments.primitives import SimulatedInteraction, LoggedInteraction, Interaction, EnvironmentFilter
+from coba.environments.primitives import Environment, Interaction
+from coba.environments.logged.primitives import LoggedInteraction
+from coba.environments.simulated.primitives import SimulatedInteraction
 
-class Take(pipes.Take):
-    
-    @property
-    def params(self) -> Dict[str, Any]:
-        return { "take": self._count }
-
-    def __str__(self) -> str:
-        return str(self.params)
-
-class Reservoir(pipes.Reservoir):
-
-    def __init__(self, count: Optional[int], seed:int=1)-> None:
-        super().__init__(count, seed)
+class EnvironmentFilter(pipes.Filter[Iterable[Interaction],Iterable[Interaction]], ABC):
+    """A filter that can be applied to an Environment."""
 
     @property
+    @abstractmethod
     def params(self) -> Dict[str, Any]:
-        return { "reservoir_count": self._count, "reservoir_seed": self._seed }
+        """Parameters that describe the filter."""
+        ...
+
+    @abstractmethod
+    def filter(self, interactions: Iterable[Interaction]) -> Iterable[Interaction]:
+        """Apply a filter to an Environment's interactions."""
+        ...
 
     def __str__(self) -> str:
-        return str(self.params)
+        return str(self.params) if self.params else self.__class__.__name__
 
-class Shuffle(pipes.Shuffle):
-    
-    @property
-    def params(self) -> Dict[str, Any]:
-        return { "shuffle": self._seed }
+class Identity(pipes.Identity, EnvironmentFilter):
+    """Return whatever interactions are given to the filter."""
 
-    def __str__(self) -> str:
-        return str(self.params)
-
-class Identity(pipes.Identity):
-    
     @property
     def params(self) -> Dict[str, Any]:
         return {}
@@ -53,31 +45,70 @@ class Identity(pipes.Identity):
     def __str__(self) -> str:
         return "{ Identity }"
 
-class Sort(EnvironmentFilter):
+class FilteredEnvironment(Environment):
+    """An Environment with a sequence of filters to apply."""
 
-    def __init__(self, *keys: Union[str,int,Sequence[Union[str,int]]]) -> None:
-        
-        self._keys = []
-        
-        for key in keys:
-            if not isinstance(key, collections.abc.Sequence) or isinstance(key,str):
-                self._keys.append(key)
-            else:
-                self._keys.extend(key)
+    def __init__(self, environment: Environment, *filters: EnvironmentFilter):
+        """Instantiate a FilteredEnvironment.
+
+        Args:
+            environment: The environment to apply filters to.
+            *filters: The sequence of filters to apply to the environment. 
+        """
+
+        if isinstance(environment, FilteredEnvironment):
+            self._source = environment._source
+            self._filter = pipes.Pipe.join([environment._filter] + list(filters))
+        elif len(filters) > 0:
+            self._source  = environment
+            self._filter = pipes.Pipe.join(list(filters))
+        else:
+            self._source = environment
+            self._filter = Identity()
 
     @property
     def params(self) -> Dict[str, Any]:
-        return { "sort": self._keys }
+        params = self._safe_params(self._source)
+        params.update(self._safe_params(self._filter))
+        return params
 
-    def filter(self, interactions: Iterable[Interaction]) -> Iterable[Interaction]:
-        return sorted(interactions, key=lambda interaction: tuple(interaction.context[key] for key in self._keys))
+    def read(self) -> Iterable[Interaction]:
+        return self._filter.filter(self._source.read())
+
+    def _safe_params(self, obj) -> Dict[str, Any]:
+        
+        if hasattr(obj, 'params'):
+            return obj.params
+
+        if hasattr(obj, '_filters'):
+            params = {}
+            for filter in obj._filters:
+                params.update(self._safe_params(filter))
+            return params
+
+        return {}
+
+    def __str__(self) -> str:
+
+        str_source = str(self._source)
+        str_filter = str(self._filter)
+
+        return ','.join(filter(None,[str_source,str_filter])).replace('{ Identity }','').replace(',,',',').strip(',')
 
 class Scale(EnvironmentFilter):
+    """Shift and scale features to precondition them before learning."""
 
     def __init__(self, 
         shift: Union[Number,Literal["min","mean","med"]] ="min", 
         scale: Union[Number,Literal["minmax","std","iqr"]]="minmax", 
         using: Optional[int] = None):
+        """Instantiate a Scale filter.
+        
+        Args:
+            shift: The statistic to use to shift each context feature.
+            scale: The statistic to use to scale each context feature.
+            using: The number of interactions to use when calculating the necessary statistics.
+        """
 
         assert isinstance(shift,Number) or shift in ["min","mean","med"]
         assert isinstance(scale,Number) or scale in ["minmax","std","iqr"]
@@ -170,10 +201,17 @@ class Scale(EnvironmentFilter):
         return []
 
 class Impute(EnvironmentFilter):
+    """Impute missing values (nan) in Interaction contexts."""
 
     def __init__(self, 
         stat : Literal["mean","median","mode"] = "mean",
         using: Optional[int] = None):
+        """Instantiate an Impute filter.
+        
+        Args:
+            stat: The statistic to use for impuatation.
+            using: The number of interactions to use to calculate the imputation statistics.
+        """
 
         assert stat in ["mean","median","mode"]
 
@@ -225,10 +263,12 @@ class Impute(EnvironmentFilter):
             else:
                 final_context = kv_imputed_context[1]
 
-            try:
+            if isinstance(interaction, SimulatedInteraction):
                 yield SimulatedInteraction(final_context, interaction.actions, **interaction.kwargs)
-            except:
+            elif isinstance(interaction, LoggedInteraction):
                 yield LoggedInteraction(final_context, interaction.action, **interaction.kwargs)
+            else:#pragma: no cover
+                raise CobaException("Unknown interactions were given to the Impute filter.") 
 
     def _context_as_name_values(self,context) -> Sequence[Tuple[Hashable,Any]]:
         
@@ -238,9 +278,59 @@ class Impute(EnvironmentFilter):
 
         return []
 
+class Sparse(EnvironmentFilter):
+    """Sparsify an environment's feature representation. 
+    
+    This has little utility beyond debugging.
+    """
+
+    def __init__(self, context:bool = True, action:bool = False):
+        """Instantiate a Sparse filter.
+
+        Args:
+            context: If True then contexts should be made sparse otherwise leave them alone.
+            action: If True then actions should be made sparse otherwise leave them alone.
+        """
+        
+        self._context = context
+        self._action  = action
+
+    @property
+    def params(self) -> Dict[str, Any]:
+        return { "sparse_C": self._context, "sparse_A": self._action }
+
+    def filter(self, interactions: Iterable[Interaction]) -> Iterable[Interaction]:
+
+        for interaction in interactions:
+
+            sparse_context = self._make_sparse(interaction.context) if self._context else interaction.context
+
+            if hasattr(interaction, 'actions'):
+                sparse_actions = list(map(self._make_sparse,interaction.actions)) if self._action else interaction.actions
+                yield SimulatedInteraction(sparse_context, sparse_actions, **interaction.kwargs)
+            else:
+                sparse_action = self._make_sparse(interaction.action) if self._action else interaction.action
+                yield LoggedInteraction(sparse_context, sparse_action, **interaction.kwargs)
+
+    def _make_sparse(self, value) -> Optional[dict]:
+        if isinstance(value,dict) or value is None:
+            return value
+        if isinstance(value,(list,tuple)):
+            return dict(enumerate(value))
+        return {0:value}
+
 class Cycle(EnvironmentFilter):
+    """Cycle all rewards associated with actions by one place.
+    
+    This filter is useful for testing an algorithms response to a non-stationary shock.
+    """
 
     def __init__(self, after:int = 0):
+        """Instantiate a Cycle filter.
+        
+        Args:
+            after: How many interactions should be seen before applying the cycle filter.
+        """
         self._after = after
 
     @property
@@ -261,6 +351,8 @@ class Cycle(EnvironmentFilter):
             yield SimulatedInteraction(interaction.context, interaction.actions, **kwargs)
 
 class Binary(EnvironmentFilter):
+    """Binarize all rewards to either 1 (max rewards) or 0 (all others)."""
+
     @property
     def params(self) -> Dict[str, Any]:
         return { "binary": True }
@@ -275,9 +367,87 @@ class Binary(EnvironmentFilter):
 
             yield SimulatedInteraction(interaction.context, interaction.actions, **kwargs)
 
-class ToWarmStart(EnvironmentFilter):
+class Take(pipes.Take, EnvironmentFilter):
+    """Take a fixed number of interactions from an Environment."""
+    
+    @property
+    def params(self) -> Dict[str, Any]:
+        return { "take": self._count }
+
+    def __str__(self) -> str:
+        return str(self.params)
+
+class Reservoir(pipes.Reservoir, EnvironmentFilter):
+    """Take a fixed number of random interactions from an Environment.
+    
+    Remarks:
+        We use Algorithm L as described by Kim-Hung Li. (1994) to take a fixed number of random items.
+
+    References:
+        Kim-Hung Li. 1994. Reservoir-sampling algorithms of time complexity O(n(1 + log(N/n))). 
+        ACM Trans. Math. Softw. 20, 4 (Dec. 1994), 481–493. DOI:https://doi.org/10.1145/198429.198435
+    """
+
+    def __init__(self, count: Optional[int], seed:int=1)-> None:
+        """Instantiate a Reservoir filter.
+
+        Args:
+            count: The number of random interactions we'd like to take.
+            seed: A random seed that controls which interactions are taken.
+        """
+        super().__init__(count, seed)
+
+    @property
+    def params(self) -> Dict[str, Any]:
+        return { "reservoir_count": self._count, "reservoir_seed": self._seed }
+
+    def __str__(self) -> str:
+        return str(self.params)
+
+class Shuffle(pipes.Shuffle, EnvironmentFilter):
+    """Shuffle a sequence of Interactions in an Environment."""
+
+    @property
+    def params(self) -> Dict[str, Any]:
+        return { "shuffle": self._seed }
+
+    def __str__(self) -> str:
+        return str(self.params)
+
+class Sort(EnvironmentFilter):
+    """Sort a sequence of Interactions in an Environment."""
+
+    def __init__(self, *keys: Union[str,int,Sequence[Union[str,int]]]) -> None:
+        """Instantiate a Sort filter.
+
+        Args:
+            *keys: The context items that should be sorted on.
+        """
+        self._keys = []
+        
+        for key in keys:
+            if not isinstance(key, collections.abc.Sequence) or isinstance(key,str):
+                self._keys.append(key)
+            else:
+                self._keys.extend(key)
+
+    @property
+    def params(self) -> Dict[str, Any]:
+        return { "sort": self._keys }
+
+    def filter(self, interactions: Iterable[Interaction]) -> Iterable[Interaction]:
+        return sorted(interactions, key=lambda interaction: tuple(interaction.context[key] for key in self._keys))
+
+class WarmStart(EnvironmentFilter):
+    """Turn a SimulatedEnvironment into a WarmStartEnvironment."""
 
     def __init__(self, n_warmstart:int, seed:int = 1):
+        """Instantiate a WarmStart filter.
+        
+        Args:
+            n_warmstart: The number of interactions that should be turned into LoggedInteractions.
+            seed: The random number seed that determines the random logging policy for LoggedInteractions.
+        """
         self._n_warmstart = n_warmstart
         self._seed = seed
 
@@ -312,34 +482,3 @@ class ToWarmStart(EnvironmentFilter):
             kwargs["reward"] = interaction.kwargs["rewards"][selected_index]
 
         return LoggedInteraction(interaction.context, selected_action, **kwargs)
-
-class Sparse(EnvironmentFilter):
-    
-    def __init__(self, context:bool = True, action:bool = False):
-
-        self._context = context
-        self._action  = action
-
-    @property
-    def params(self) -> Dict[str, Any]:
-        return { "sparse_C": self._context, "sparse_A": self._action }
-
-    def filter(self, interactions: Iterable[Interaction]) -> Iterable[Interaction]:
-
-        for interaction in interactions:
-
-            sparse_context = self._make_sparse(interaction.context) if self._context else interaction.context
-
-            if hasattr(interaction, 'actions'):
-                sparse_actions = list(map(self._make_sparse,interaction.actions)) if self._action else interaction.actions
-                yield SimulatedInteraction(sparse_context, sparse_actions, **interaction.kwargs)
-            else:
-                sparse_action = self._make_sparse(interaction.action) if self._action else interaction.action
-                yield LoggedInteraction(sparse_context, sparse_action, **interaction.kwargs)
-
-    def _make_sparse(self, value) -> Optional[dict]:
-        if isinstance(value,dict) or value is None:
-            return value
-        if isinstance(value,(list,tuple)):
-            return dict(enumerate(value))
-        return {0:value}
