@@ -1,6 +1,6 @@
 from pathlib import Path
 from typing import Sequence, Optional
-from coba.backport import Literal
+from coba.backports import Literal
 
 from coba.pipes import Pipe, Foreach
 from coba.learners import Learner
@@ -10,26 +10,26 @@ from coba.contexts import CobaContext, ExceptLog, StampLog, NameLog, DecoratedLo
 
 from coba.experiments.process import CreateWorkItems,  RemoveFinished, ChunkByTask, ChunkBySource, ProcessWorkItems
 from coba.experiments.tasks   import EnvironmentTask, EvaluationTask, LearnerTask
-from coba.experiments.tasks   import SimpleLearnerTask, SimpleEnvironmentTask, OnPolicyEvaluationTask
+from coba.experiments.tasks   import SimpleLearnerTask, SimpleEnvironmentTask, OnlineOnPolicyEvalTask
 from coba.experiments.results import Result, TransactionIO
 
 class Experiment:
-    """An Experiment which uses environments and learners to calculate performance statistics."""
+    """An Experiment using a collection of environments and learners."""
 
     def __init__(self,
         environments    : Sequence[Environment],
         learners        : Sequence[Learner],
         learner_task    : LearnerTask     = SimpleLearnerTask(),
         environment_task: EnvironmentTask = SimpleEnvironmentTask(),
-        evaluation_task : EvaluationTask  = OnPolicyEvaluationTask()) -> None:
+        evaluation_task : EvaluationTask  = OnlineOnPolicyEvalTask()) -> None:
         """Instantiate an Experiment.
 
         Args:
             environments: The collection of environments to use in the experiment.
             learners: The collection of learners to use in the experiment.
             learner_task: A task which describes a learner.
-            environment_task: A task which describes an environment and its interactions.
-            evaluation_task: A task which evaluates performance of a learner on an environment's interactions.
+            environment_task: A task which describes an environment.
+            evaluation_task: A task which evaluates a learner on an environment.
         """
 
         self._environments     = environments
@@ -38,51 +38,59 @@ class Experiment:
         self._environment_task = environment_task
         self._evaluation_task  = evaluation_task
 
-        self._processes       : Optional[int] = None
-        self._maxtasksperchild: Optional[int] = None
-        self._chunk_by        : Optional[str] = None
+        self._processes        : Optional[int] = None
+        self._maxchunksperchild: Optional[int] = None
+        self._chunk_by         : Optional[str] = None
 
     def config(self, 
         processes: int = None,
         chunk_by: Literal['source','task'] = None,
-        maxtasksperchild: Optional[int] = None) -> 'Experiment':
-        """Configure how the experiment will be executed. A value of `None` for any item means the global config will be used.
+        maxchunksperchild: Optional[int] = None) -> 'Experiment':
+        """Configure how the experiment will be executed. 
+        
+        A value of `None` for any item means the CobaContext.experiment will be used.
 
         Args:
-            chunk_by: Determines how tasks are chunked for processing.
-            processes: Determines how many processes to use when processing task chunks.
-            maxtasksperchild: Determines how many tasks each process will complete before being restarted. A value of 0 means infinite.
+            chunk_by: The method for chunking tasks before processing.
+            processes: The number of processes to create for evaluating the experiment.
+            maxchunkssperchild: The number of chunks each process evaluate before being restarted. A value of 0 means infinite.
         """
 
         assert chunk_by is None or chunk_by in ['task', 'source'], "The given chunk_by value wasn't recognized. Allowed values are 'task', 'source' and 'none'"
         assert processes is None or processes > 0, "The given number of processes is invalid. Must be greater than 0."
-        assert maxtasksperchild is None or maxtasksperchild >= 0, "The given number of taks per child is invalid. Must be greater than or equal to 0 (0 for infinite)."
+        assert maxchunksperchild is None or maxchunksperchild >= 0, "The given number of taks per child is invalid. Must be greater than or equal to 0 (0 for infinite)."
 
-        self._chunk_by         = chunk_by
-        self._processes        = processes
-        self._maxtasksperchild = maxtasksperchild
+        self._chunk_by          = chunk_by
+        self._processes         = processes
+        self._maxchunksperchild = maxchunksperchild
 
         return self
 
     @property
     def chunk_by(self) -> str:
+        """The method for chunking tasks before sending them to processes for execution. 
+
+        This option is only relevant if the experiment is being executed on multiple processes.
+        """
         return self._chunk_by if self._chunk_by is not None else CobaContext.experiment.chunk_by
 
     @property
     def processes(self) -> int:
+        """The number of processes to use when evaluating the experiment."""
         return self._processes if self._processes is not None else CobaContext.experiment.processes
 
     @property
-    def maxtasksperchild(self) -> int:
-        return self._maxtasksperchild if self._maxtasksperchild is not None else CobaContext.experiment.maxtasksperchild
+    def maxchunksperchild(self) -> int:
+        """The number of chunks to perform per process before restarting an evaluation process."""
+        return self._maxchunksperchild if self._maxchunksperchild is not None else CobaContext.experiment.maxchunksperchild
 
     def evaluate(self, result_file:str = None) -> Result:
         """Evaluate the experiment and return the results.
 
         Args:
-            result_file: The file we'd like to use for writing/restoring results experiments.
+            result_file: The file for writing and restoring results .
         """
-        cb, mp, mt = self.chunk_by, self.processes, self.maxtasksperchild
+        cb, mp, mt = self.chunk_by, self.processes, self.maxchunksperchild
 
         if mp > 1 or mt != 0:
             CobaContext.logger = DecoratedLogger([ExceptLog()], CobaContext.logger, [NameLog(), StampLog()])
@@ -120,4 +128,4 @@ class Experiment:
         if isinstance(CobaContext.logger, DecoratedLogger):
             CobaContext.logger = CobaContext.logger.undecorate()
 
-        return sink.read()
\ No newline at end of file
+        return sink.read()
