@@ -1,5 +1,3 @@
-"""Simple one-off utility methods with no clear home."""
-
 import warnings
 import importlib
 
@@ -28,7 +26,7 @@ class PackageChecker:
             at https://github.com/scikit-learn/scikit-learn/blob/master/sklearn/utils/__init__.py
         """
         try:
-            importlib.import_module('matplotlib')
+            importlib.import_module('matplotlib.pyplot')
         except ImportError:
             PackageChecker._handle_import_error(caller_name, "matplotlib")
 
@@ -120,4 +118,4 @@ class KeyDefaultDict(defaultdict):
         else:
             value = self.default_factory(key)
             self[key] = value
-            return value
\ No newline at end of file
+            return value
